use DataScience
go

-- populate dbo.source_data for [dbo].[usp_ema_computer_with_dec_vals] proc
-- based on dbo.symbol_date
drop table if exists dbo.source_data

select * into dbo.source_data from dbo.symbol_date

alter table source_data
add constraint pk_symbol_date_source_data primary key (symbol,date);


-- create fast_ema and slow_ema tables
drop table if exists fast_ema
go

create table dbo.fast_ema (
 date date
,symbol nvarchar(10)
,[close] dec(19,4)
,period_length dec(19,4)
,ema dec(19,4)
)

drop table if exists dbo.slow_ema
go

create table dbo.slow_ema (
 date date
,symbol nvarchar(10)
,[close] dec(19,4)
,period_length dec(19,4)
,ema dec(19,4)
)


-- compute fast_ema and slow_ema values for MACD line
-- @alpha parameter = 2/13 for @period = 12
-- @alpha parameter = 2/27 for @period = 26
-- compute @alpha to rounded to 4 places after the decimal
insert into fast_ema
exec usp_ema_computer_with_dec_vals @symbol = 'SPY', @period = 12, @alpha = 0.1538

insert into slow_ema
exec usp_ema_computer_with_dec_vals @symbol = 'SPY', @period = 26, @alpha = 0.0741

-- echo fast_ema
select * from fast_ema

-- echo slow_ema
select * from slow_ema


-- compute MACD based on fast_ema - slow_ema
drop table  if exists #MACD
go

create table #MACD(
 symbol nvarchar(10) not null
,date date not null
,MACD dec(19,4) not null
)

insert into #MACD
-- remove initial MACD null column value
select 
 fast_ema.symbol
,fast_ema.date
,(fast_ema.ema - slow_ema.ema) MACD
from fast_ema inner join slow_ema on fast_ema.date = slow_ema.date 
where (fast_ema.ema - slow_ema.ema) is not null 


alter table #MACD
add constraint pk_symbol_date_#MACD primary key (symbol,date)

select * from #MACD

--------------------------------------------------------------------------------------------------------------------
use DataScience
go

-- compute Signal as an exponential moving average 
-- based on the MACD column values in the #MACD table

-- populate a fresh copy of dbo.source_data based on the #MACD table
-- for the usp_ema_computer_with_dec_vals proc
drop table if exists dbo.source_data

-- create and populate dbo.source_data from #MACD
-- designate an alias name [close] for the MACD column from the #MACD table  
-- for use in the usp_ema_computer_with_dec_vals proc
select 
 symbol
,date
,MACD [close] 
into dbo.source_data 
from #MACD

alter table source_data
add constraint pk_symbol_date_source_data primary key (symbol,date);

-- compute #Signal as an ema with a period length for the third MACD setting 
-- default value is 9

drop table  if exists #Signal
go

create table #Signal (
 date date
,symbol nvarchar(10)
,[MACD]  dec(19,4)
,period_length dec(19,4)
,Signal dec(19,4)
)

-- Compute Signal column values as an exponential moving average
insert into #Signal
exec usp_ema_computer_with_dec_vals @symbol = 'SPY', @period = 9, @alpha = 0.2

-- display #Signal table rows with Signal column values
-- computed from a 9 period length exponential moving average of MACD values
select * from #Signal order by date

-- left join #Signal to symbol_date to display
-- close, MACD, and Signal values
select 
 symbol_date.symbol
,symbol_date.date
,[close]
,#Signal.MACD
,#Signal.Signal
from symbol_date
left join #Signal
on symbol_date.symbol = #Signal.symbol
and symbol_date.date = #Signal.date
where symbol_date.symbol = 'SPY'
 and #Signal.Signal is not null

---------------------------------------------------------------------------------------
use DataScience
go

-- compute Histogram as MACD - Signal 
-- from for_Histogram
select 
 for_Histogram.*
,(for_Histogram.MACD - for_Histogram.Signal) Histogram
 from
 (
 -- left join #Signal to symbol_date to display
-- close, MACD, and Signal values
select 
 symbol_date.symbol
,symbol_date.date
,[close]
,#Signal.MACD
,#Signal.Signal
from symbol_date
left join #Signal
on symbol_date.symbol = #Signal.symbol
and symbol_date.date = #Signal.date
where symbol_date.symbol = 'SPY'
 ) for_Histogram
